@@ -1,5 +1,5 @@
 package Treex::Core::Node;
-$Treex::Core::Node::VERSION = '2.20160630';
+$Treex::Core::Node::VERSION = '2.20201228';
 use namespace::autoclean;
 
 use Moose;
@@ -92,7 +92,15 @@ sub _not_equals {
 
 sub _index_my_id {
     my $self = shift;
-    $self->get_document->index_node_by_id( $self->id, $self );
+    # It is possible that get_document is undefined at the moment (for example
+    # if we are reading one large document and splitting it to many small documents).
+    # Then there is nothing to index (and everything will be probably indexed
+    # later when the document is created).
+    my $document = $self->get_document();
+    if (defined($document))
+    {
+        $document->index_node_by_id( $self->id, $self );
+    }
     return;
 }
 
@@ -270,7 +278,7 @@ sub remove {
     }
     my $root     = $self->get_root();
     my $document = $self->get_document();
-    
+
     my @children = $self->get_children();
     if (@children){
         my $what_to_do = 'remove';
@@ -321,7 +329,7 @@ sub get_referencing_nodes {
     if ((defined $lang) && (defined $sel)) {
         my @ref_filtered_by_tree;
         if ($sel eq q() ) {
-            @ref_filtered_by_tree = grep { /(a|t)\_tree\-$lang\-.+/; }@{ $refs->{$type} };            
+            @ref_filtered_by_tree = grep { /(a|t)\_tree\-$lang\-.+/; }@{ $refs->{$type} };
         }
         else {
             @ref_filtered_by_tree = grep { /(a|t)\_tree\-$lang\_$sel\-.+/; }@{ $refs->{$type} };
@@ -557,9 +565,10 @@ sub _check_switches {
         if $arg_ref->{first_only} && $arg_ref->{last_only};
 
     # Check for explicit "ordered" when not needed (possible typo)
-    log_warn('Specifying (first|last|preceding|following)_only implies ordered.')
-        if $arg_ref->{ordered}
-            && any { $arg_ref->{ $_ . '_only' } } qw(first last preceding following);
+    # DZ: Turning off. This is an annoying warning when I want to explicitly say in my code that the result will be ordered and accessing the first/last item is meaningful.
+    #log_warn('Specifying (first|last|preceding|following)_only implies ordered.')
+    #    if $arg_ref->{ordered}
+    #        && any { $arg_ref->{ $_ . '_only' } } qw(first last preceding following);
 
     # Check for unknown switches
     my $unknown = first { $_ !~ $_SWITCHES_REGEX } keys %{$arg_ref};
@@ -711,6 +720,70 @@ sub get_address {
     return "$file##$position.$id";
 }
 
+# The MISC attributes from CoNLL-U files are stored as wild attributes.
+# These methods should be in a Universal Dependencies related role but we don't have one.
+# get_misc() returns a list of MISC attributes (possibly empty list)
+sub get_misc
+{
+    my $self = shift;
+    my @misc;
+    my $wild = $self->wild();
+    if (exists($wild->{misc}) && defined($wild->{misc}))
+    {
+        @misc = split(/\|/, $wild->{misc});
+    }
+    return @misc;
+}
+
+# set_misc() takes a list of MISC attributes (possibly empty list)
+sub set_misc
+{
+    my $self = shift;
+    my @misc = @_;
+    my $wild = $self->wild();
+    if (scalar(@misc) > 0)
+    {
+        $wild->{misc} = join('|', @misc);
+    }
+    else
+    {
+        delete($wild->{misc});
+    }
+}
+
+# set_misc_attr() takes an attribute name and value; assumes that MISC elements are attr=value pairs; replaces first or pushes at the end
+sub set_misc_attr
+{
+    my $self = shift;
+    my $attr = shift;
+    my $value = shift;
+    if (defined($attr) && defined($value))
+    {
+        my @misc = $self->get_misc();
+        my $found = 0;
+        for(my $i = 0; $i <= $#misc; $i++)
+        {
+            if ($misc[$i] =~ m/^(.+?)=(.+)$/ && $1 eq $attr)
+            {
+                if ($found)
+                {
+                    splice(@misc, $i--, 1);
+                }
+                else
+                {
+                    $misc[$i] = "$attr=$value";
+                    $found = 1;
+                }
+            }
+        }
+        if (!$found)
+        {
+            push(@misc, "$attr=$value");
+        }
+        $self->set_misc(@misc);
+    }
+}
+
 # Empty DESTROY method is a hack to get rid of the "Deep recursion warning"
 # in Treex::PML::Node::DESTROY and MooseX::NonMoose::Meta::Role::Class::_check_superclass_destructor.
 # Without this hack, you get the warning after creating a node with 99 or more children.
@@ -883,7 +956,7 @@ sub descendants {
 # TODO: How to do this in an elegant way?
 # Unless we find a better way, we must disable two perlcritics
 package Treex::Core::Node::Removed;    ## no critic (ProhibitMultiplePackages)
-$Treex::Core::Node::Removed::VERSION = '2.20160630';
+$Treex::Core::Node::Removed::VERSION = '2.20201228';
 use Treex::Core::Log;
 
 sub AUTOLOAD {                         ## no critic (ProhibitAutoloading)
@@ -959,7 +1032,7 @@ Treex::Core::Node - smallest unit that holds information in Treex
 
 =head1 VERSION
 
-version 2.20160630
+version 2.20201228
 
 =head1 DESCRIPTION
 
