@@ -1,5 +1,5 @@
 package Treex::Core::Node::T;
-$Treex::Core::Node::T::VERSION = '2.20160630';
+$Treex::Core::Node::T::VERSION = '2.20201228';
 use namespace::autoclean;
 
 use Moose;
@@ -161,69 +161,110 @@ sub get_anodes {
 
 #------------ coreference and bridging nodes -------------------
 
+sub get_appos_expansion {
+    my ($self, $arg_ref ) = @_;
+    if ($self->is_coap_root && $self->functor eq "APPS") {
+        return ($self->get_coap_members({direct_only => 1}), $arg_ref->{with_appos_root} ? $self : ());
+    }
+    else {
+        my $par = $self->get_parent;
+        if (defined $par && !$par->is_root && $par->is_coap_root && $par->functor eq "APPS" && $self->is_member ) {
+            return ($par->get_coap_members({direct_only => 1}), $arg_ref->{with_appos_root} ? $par : ());
+        }
+    }
+    return $self;
+}
+
+# types are not allowed for the time being
+sub _unfold_appos {
+    my ( $self, $type ) = @_;
+
+    my @appos_nodes_not_self = grep {$_ != $self} $self->get_appos_expansion({with_appos_root => 1});
+
+    my @member_antes = ();
+    # type = { text, gram, all }
+    if ($type eq "gram") {
+        @member_antes = map {$_->get_coref_gram_nodes({appos_aware => 0})} @appos_nodes_not_self;
+    }
+    elsif ($type eq "text") {
+        @member_antes = map {$_->get_coref_text_nodes({appos_aware => 0})} @appos_nodes_not_self;
+    }
+    else {
+        @member_antes = map {$_->get_coref_nodes({appos_aware => 0})} @appos_nodes_not_self;
+    }
+    return @member_antes;
+}
+
+sub _replace_appos_antes {
+    my (@antes) = @_;
+    my @new_antes = map {$_->get_appos_expansion({with_appos_root => 0})} @antes;
+    my %seen = ();
+    my @unique_antes = grep { !$seen{$_->id}++ } @new_antes;
+    return @unique_antes;
+}
+
+sub _get_coref_nodes {
+    my ($self, $type, $arg_ref) = @_;
+
+    my $appos_aware = $arg_ref->{appos_aware} // 1;
+    my $with_types = $arg_ref->{with_types} // 0;
+    delete $arg_ref->{$_} foreach (qw/appos_aware with_types/);
+
+    my @antes = ();
+    my @types = ();
+
+    if ($type ne 'text') {
+        my @gram_nodes = $self->_get_node_list('coref_gram.rf');
+        push @antes, @gram_nodes;
+        push @types, map {undef} @gram_nodes;
+    }
+    if ($type ne 'gram') {
+        # textual coreference in PDT2.0 and 2.5 style
+        my @text_nodes = $self->_get_node_list('coref_text.rf');
+        push @antes, @text_nodes;
+        push @types, map {undef} @text_nodes;
+        # textual coreference in PDT3.0 style
+        my $pdt30_text_coref_rf = $self->get_attr('coref_text') // [];
+        @text_nodes = map {$self->get_document->get_node_by_id( $_->{'target_node.rf'} )} @$pdt30_text_coref_rf;
+        my @text_types = map { $_->{'type'} } @$pdt30_text_coref_rf;
+        push @antes, @text_nodes;
+        push @types, @text_types;
+    }
+
+    # for the time being, apposition-aware handling is disabled if types are demanded
+    if (!$with_types && $appos_aware) {
+        push @antes, $self->_unfold_appos($type);
+        @antes = _replace_appos_antes(@antes);
+    }
+
+    my @filtered_antes = $self->_process_switches( $arg_ref, @antes );
+    return @filtered_antes if (!$with_types);
+
+    # return both nodes and types (as list references - similar to alignments)
+    my %node_id_to_index = map {$antes[$_]->id => $_} 0 .. $#antes;
+    my @filtered_types = map {
+        my $idx = $node_id_to_index{$_->id};
+        defined $idx ? $types[$idx] : undef
+    } @filtered_antes;
+    return (\@filtered_antes, \@filtered_types);
+}
+
+
 sub get_coref_nodes {
     my ( $self, $arg_ref ) = @_;
-    
-    # process coreference parameters
-    my $with_types = $arg_ref->{with_types};
-    delete $arg_ref->{with_types};
-    
-    my @gram_nodes = $self->_get_node_list('coref_gram.rf');
-    
-    # textual coreference in PDT2.0 and 2.5 style
-    my @text_nodes = $self->_get_node_list('coref_text.rf');
-    return $self->_process_switches( $arg_ref, (@gram_nodes, @text_nodes) ) if (@text_nodes);
-
-    # textual coreference in PDT3.0 style
-    my $pdt30_text_coref_rf = $self->get_attr('coref_text') // [];
-    my @pdt30_gram_coref = map {{'target_node.rf' => $_->id, 'type' => undef}} @gram_nodes;
-    return $self->_get_pdt30_coref([@pdt30_gram_coref, @$pdt30_text_coref_rf], $with_types, $arg_ref);
+    return $self->_get_coref_nodes('all', $arg_ref);
 }
 
 sub get_coref_gram_nodes {
     my ( $self, $arg_ref ) = @_;
-    return $self->_get_node_list( 'coref_gram.rf', $arg_ref );
+    return $self->_get_coref_nodes('gram', $arg_ref);
 }
 
 sub get_coref_text_nodes {
     my ( $self, $arg_ref ) = @_;
-    
-    # process coreference parameters
-    my $with_types = $arg_ref->{with_types};
-    delete $arg_ref->{with_types};
-    
-    # textual coreference in PDT2.0 and 2.5 style
-    my @nodes = $self->_get_node_list( 'coref_text.rf', $arg_ref );
-    return @nodes if (@nodes);
-    
-    # textual coreference in PDT3.0 style
-    my $pdt30_coref_rf = $self->get_attr('coref_text') // [];
-
-    return $self->_get_pdt30_coref($pdt30_coref_rf, $with_types, $arg_ref);
+    return $self->_get_coref_nodes('text', $arg_ref);
 }
 
-sub _get_pdt30_coref {
-    my ($self, $coref_rf, $with_types, $arg_ref) = @_;
-    
-    my $document = $self->get_document;
-    
-    my @nodes = map {$document->get_node_by_id( $_->{'target_node.rf'} )} @$coref_rf;
-    ## get_node_by_id() will fatally fail if target is not defined!
-    #my @targetrfs = grep {defined($_)} map {$_->{'target_node.rf'}} @{$coref_rf};
-    #my @nodes = map {$document->get_node_by_id($_)} @targetrfs;
-    
-    my @filtered_nodes = $self->_process_switches( $arg_ref, @nodes );
-    return @filtered_nodes if (!$with_types);
-    
-    # return both nodes and types (as list references - similar to alignments)
-    my %node_id_to_index = map {$nodes[$_]->id => $_} 0 .. $#nodes;
-    my @types = map { $_->{'type'} } @$coref_rf;
-    my @filtered_types = map {
-        my $idx = $node_id_to_index{$_->id};
-        defined $idx ? $types[$idx] : undef
-    } @filtered_nodes;
-    return (\@filtered_nodes, \@filtered_types);
-}
 
 # it doesn't return a complete chain, just the members which are accessible
 # from the current node
@@ -233,11 +274,12 @@ sub get_coref_chain {
 
     my %visited_nodes = ();
     my @nodes;
-    my @queue = ( $self->_get_node_list('coref_gram.rf'), $self->_get_node_list('coref_text.rf') );
+    my %sub_args = map {$_ => $arg_ref->{$_}} qw/appos_aware with_types/;
+    my @queue = $self->get_coref_nodes(\%sub_args);
     while ( my $node = shift @queue ) {
         $visited_nodes{$node} = 1;
         push @nodes, $node;
-        my @antes = ( $node->_get_node_list('coref_gram.rf'), $node->_get_node_list('coref_text.rf') );
+        my @antes = $node->get_coref_nodes(\%sub_args);
         foreach my $ante (@antes) {
             if ( !defined $visited_nodes{$ante} ) {
                 push @queue, $ante;
@@ -248,14 +290,43 @@ sub get_coref_chain {
     return $self->_process_switches( $arg_ref, @nodes );
 }
 
+sub select_node_if_apps {
+    my ($node) = @_;
+    my $par = $node->get_parent;
+    if (defined $par && !$par->is_root && $par->is_coap_root && $par->functor eq "APPS" && $node->is_member ) {
+        return $par;
+    }
+    return $node;
+}
+
+sub _add_coref_nodes {
+    my ($self, $type, @antes) = @_;
+
+    # if called on a member of an apposition, call it on the apposition root
+    my $anaph = select_node_if_apps($self);
+    if ($anaph != $self) {
+        return $anaph->_add_coref_nodes($type, @antes);
+    }
+
+    # antes must be apposition roots if in apposition
+    @antes = map {select_node_if_apps($_)} @antes;
+
+    # avoid link repeating and self-reference
+    my %seen;
+    @antes = grep {$_ != $self} grep {!$seen{$_->id}++} @antes;
+    return if (!@antes);
+
+    $self->_add_to_node_list( "coref_$type.rf", @antes );
+}
+
 sub add_coref_gram_nodes {
     my $self = shift;
-    return $self->_add_to_node_list( 'coref_gram.rf', @_ );
+    $self->_add_coref_nodes('gram', @_);
 }
 
 sub add_coref_text_nodes {
     my $self = shift;
-    return $self->_add_to_node_list( 'coref_text.rf', @_ );
+    $self->_add_coref_nodes('text', @_);
 }
 
 sub remove_coref_nodes {
@@ -278,7 +349,7 @@ sub get_bridging_nodes {
     my ($self, $arg_ref) = @_;
     my $bridging = $self->get_attr('bridging') // [];
     my $doc = $self->get_document;
-    my @nodes = map {$doc->get_node_by_id($_->{'target_node.rf'})} @$bridging; 
+    my @nodes = map {$doc->get_node_by_id($_->{'target_node.rf'})} @$bridging;
     my @types = map {$_->{'type'}} @$bridging;
     return (\@nodes, \@types);
 }
@@ -355,6 +426,7 @@ sub gram_number        { return $_[0]->get_attr('gram/number'); }
 sub gram_degcmp        { return $_[0]->get_attr('gram/degcmp'); }
 sub gram_verbmod       { return $_[0]->get_attr('gram/verbmod'); }
 sub gram_deontmod      { return $_[0]->get_attr('gram/deontmod'); }
+sub gram_factmod       { return $_[0]->get_attr('gram/factmod'); }
 sub gram_tense         { return $_[0]->get_attr('gram/tense'); }
 sub gram_aspect        { return $_[0]->get_attr('gram/aspect'); }
 sub gram_resultative   { return $_[0]->get_attr('gram/resultative'); }
@@ -367,6 +439,8 @@ sub gram_politeness    { return $_[0]->get_attr('gram/politeness'); }
 sub gram_negation      { return $_[0]->get_attr('gram/negation'); }
 sub gram_definiteness  { return $_[0]->get_attr('gram/definiteness'); }
 sub gram_diathesis     { return $_[0]->get_attr('gram/diathesis'); }
+sub gram_diatgram      { return $_[0]->get_attr('gram/diatgram'); }
+sub gram_typgroup      { return $_[0]->get_attr('gram/typgroup'); }
 
 sub set_gram_sempos        { return $_[0]->set_attr( 'gram/sempos',        $_[1] ); }
 sub set_gram_gender        { return $_[0]->set_attr( 'gram/gender',        $_[1] ); }
@@ -374,6 +448,7 @@ sub set_gram_number        { return $_[0]->set_attr( 'gram/number',        $_[1]
 sub set_gram_degcmp        { return $_[0]->set_attr( 'gram/degcmp',        $_[1] ); }
 sub set_gram_verbmod       { return $_[0]->set_attr( 'gram/verbmod',       $_[1] ); }
 sub set_gram_deontmod      { return $_[0]->set_attr( 'gram/deontmod',      $_[1] ); }
+sub set_gram_factmod       { return $_[0]->set_attr( 'gram/factmod',       $_[1] ); }
 sub set_gram_tense         { return $_[0]->set_attr( 'gram/tense',         $_[1] ); }
 sub set_gram_aspect        { return $_[0]->set_attr( 'gram/aspect',        $_[1] ); }
 sub set_gram_resultative   { return $_[0]->set_attr( 'gram/resultative',   $_[1] ); }
@@ -386,6 +461,8 @@ sub set_gram_politeness    { return $_[0]->set_attr( 'gram/politeness',    $_[1]
 sub set_gram_negation      { return $_[0]->set_attr( 'gram/negation',      $_[1] ); }
 sub set_gram_definiteness  { return $_[0]->set_attr( 'gram/definiteness',  $_[1] ); }
 sub set_gram_diathesis     { return $_[0]->set_attr( 'gram/diathesis',     $_[1] ); }
+sub set_gram_diatgram      { return $_[0]->set_attr( 'gram/diatgram',      $_[1] ); }
+sub set_gram_typgroup      { return $_[0]->set_attr( 'gram/typgroup',      $_[1] ); }
 
 #------------- valency frame reference -----
 
@@ -404,7 +481,7 @@ Treex::Core::Node::T
 
 =head1 VERSION
 
-version 2.20160630
+version 2.20201228
 
 =head1 DESCRIPTION
 
@@ -486,6 +563,25 @@ The method returns references to two lists of the equal length: the referred nod
 
 Add bridging anaphora to C<$node> of type C<$type> (to C<bridging>).
 
+=item $node->get_appos_expansion($arg_ref?)
+
+If the node is part of an apposition (no matter whether as a root
+or a member), return all the nodes that constitute the apposition.
+Only the apposition root may be excluded from the selection if the
+option C<with_appos_root> is off.
+The method is used to abstract from the technical implementation
+of appositions. So far, it is used by coreference accessors.
+
+OPTIONS
+
+=over
+
+=item with_appos_root
+
+Include the apposition root. Disabled by default.
+
+=back
+
 =back
 
 =head2 Access to source language t-layer (in MT)
@@ -546,8 +642,10 @@ Martin Popel <popel@ufal.mff.cuni.cz>
 
 Ondřej Dušek <odusek@ufal.mff.cuni.cz>
 
+Michal Novák <mnovak@ufal.mff.cuni.cz>
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2011-2012 by Institute of Formal and Applied Linguistics, Charles University in Prague
+Copyright © 2011-2016 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
 This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
